# -*- coding: utf-8 -*-
'''
Module for handling OpenStack Neutron calls

:depends:   - neutronclient Python module
:configuration: This module is not usable until the user, password, tenant, and
    auth URL are specified either in a pillar or in the minion's config file.
    For example::

        keystone.user: 'admin'
        keystone.password: 'password'
        keystone.tenant: 'admin'
        keystone.auth_url: 'http://127.0.0.1:5000/v2.0/'
        keystone.region_name: 'RegionOne'
        keystone.service_type: 'network'

    If configuration for multiple OpenStack accounts is required, they can be
    set up as different configuration profiles:
    For example::

        openstack1:
          keystone.user: 'admin'
          keystone.password: 'password'
          keystone.tenant: 'admin'
          keystone.auth_url: 'http://127.0.0.1:5000/v2.0/'
          keystone.region_name: 'RegionOne'
          keystone.service_type: 'network'

        openstack2:
          keystone.user: 'admin'
          keystone.password: 'password'
          keystone.tenant: 'admin'
          keystone.auth_url: 'http://127.0.0.2:5000/v2.0/'
          keystone.region_name: 'RegionOne'
          keystone.service_type: 'network'

    With this configuration in place, any of the neutron functions
    can make use of a configuration profile by declaring it explicitly.
    For example::

        salt '*' neutron.network_list profile=openstack1
'''

# Import python libs
from __future__ import absolute_import
import logging

# Import salt libs
try:
    import salt.utils.openstack.neutron as suoneu
    HAS_NEUTRON = True
except NameError as exc:
    HAS_NEUTRON = False

# Get logging started
log = logging.getLogger(__name__)

# Function alias to not shadow built-ins
__func_alias__ = {
    'list_': 'list'
}


def __virtual__():
    '''
    Only load this module if neutron
    is installed on this minion.
    '''
    return HAS_NEUTRON


__opts__ = {}


def _auth(profile=None):
    '''
    Set up neutron credentials
    '''
    if profile:
        credentials = __salt__['config.option'](profile)
        user = credentials['keystone.user']
        password = credentials['keystone.password']
        tenant = credentials['keystone.tenant']
        auth_url = credentials['keystone.auth_url']
        region_name = credentials.get('keystone.region_name', None)
        service_type = credentials['keystone.service_type']
    else:
        user = __salt__['config.option']('keystone.user')
        password = __salt__['config.option']('keystone.password')
        tenant = __salt__['config.option']('keystone.tenant')
        auth_url = __salt__['config.option']('keystone.auth_url')
        region_name = __salt__['config.option']('keystone.region_name')
        service_type = __salt__['config.option']('keystone.service_type')

    kwargs = {
        'username': user,
        'password': password,
        'tenant_name': tenant,
        'auth_url': auth_url,
        'region_name': region_name,
        'service_type': service_type
    }

    return suoneu.SaltNeutron(**kwargs)


def get_quotas_tenant(profile=None):
    '''
    Fetches tenant info in server's context for following quota operation

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.get_quotas_tenant
        salt '*' neutron.get_quotas_tenant profile=openstack1

    :param profile: Profile to build on (Optional)
    :return: Quotas information
    '''

    conn = _auth(profile)
    return conn.get_quotas_tenant()


def list_quotas(profile=None):
    '''
    Fetches all tenants quotas

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.list_quotas
        salt '*' neutron.list_quotas profile=openstack1

    :param profile: Profile to build on (Optional)
    :return: List of quotas
    '''
    conn = _auth(profile)
    return conn.list_quotas()


def show_quota(tenant_id, profile=None):
    '''
    Fetches information of a certain tenant's quotas

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.show_quota tenant-id
        salt '*' neutron.show_quota tenant-id profile=openstack1

    :param tenant_id: ID of tenant
    :param profile: Profile to build on (Optional)
    :return: Quota information
    '''
    conn = _auth(profile)
    return conn.show_quota(tenant_id)


def update_quota(tenant_id,
                 subnet=None,
                 router=None,
                 network=None,
                 floatingip=None,
                 port=None,
                 security_group=None,
                 security_group_rule=None,
                 profile=None):
    '''
    Update a tenant's quota

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.update_quota tenant-id subnet=40 router=50
                                    network=10 floatingip=30 port=30

    :param tenant_id: ID of tenant
    :param subnet: Value of subnet quota (Optional)
    :param router: Value of router quota (Optional)
    :param network: Value of network quota (Optional)
    :param floatingip: Value of floatingip quota (Optional)
    :param port: Value of port quota (Optional)
    :param security_group: Value of security group (Optional)
    :param security_group_rule: Value of security group rule (Optional)
    :param profile: Profile to build on (Optional)
    :return: Value of updated quota
    '''
    conn = _auth(profile)
    return conn.update_quota(tenant_id, subnet, router, network,
                             floatingip, port, security_group,
                             security_group_rule)


def delete_quota(tenant_id, profile=None):
    '''
    Delete the specified tenant's quota value

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.update_quota tenant-id
        salt '*' neutron.update_quota tenant-id profile=openstack1

    :param tenant_id: ID of tenant to quota delete
    :param profile: Profile to build on (Optional)
    :return: True(Delete succeed) or False(Delete failed)
    '''
    conn = _auth(profile)
    return conn.delete_quota(tenant_id)


def list_extensions(profile=None):
    '''
    Fetches a list of all extensions on server side

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.list_extensions
        salt '*' neutron.list_extensions profile=openstack1

    :param profile: Profile to build on (Optional)
    :return: List of extensions
    '''
    conn = _auth(profile)
    return conn.list_extensions()


def list_ports(profile=None):
    '''
    Fetches a list of all networks for a tenant

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.list_ports
        salt '*' neutron.list_ports profile=openstack1

    :param profile: Profile to build on (Optional)
    :return: List of port
    '''
    conn = _auth(profile)
    return conn.list_ports()


def show_port(port, profile=None):
    '''
    Fetches information of a certain port

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.show_port port-id
        salt '*' neutron.show_port port-id profile=openstack1

    :param port: ID or name of port to look up
    :param profile: Profile to build on (Optional)
    :return: Port information
    '''
    conn = _auth(profile)
    return conn.show_port(port)


def create_port(name,
                network,
                device_id=None,
                admin_state_up=True,
                profile=None):
    '''
    Creates a new port

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.create_port network-name port-name

    :param name: Name of port to create
    :param network: Network name or ID
    :param device_id: ID of device (Optional)
    :param admin_state_up: Set admin state up to true or false,
            default: true (Optional)
    :param profile: Profile to build on (Optional)
    :return: Created port information
    '''
    conn = _auth(profile)
    return conn.create_port(name, network, device_id, admin_state_up)


def update_port(port, name, admin_state_up=True, profile=None):
    '''
    Updates a port

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.update_port port-name network-name new-port-name

    :param port: Port name or ID
    :param name: Name of this port
    :param admin_state_up: Set admin state up to true or false,
            default: true (Optional)
    :param profile: Profile to build on (Optional)
    :return: Value of updated port information
    '''
    conn = _auth(profile)
    return conn.update_port(port, name, admin_state_up)


def delete_port(port, profile=None):
    '''
    Deletes the specified port

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.delete_network port-name
        salt '*' neutron.delete_network port-name profile=openstack1

    :param port: port name or ID
    :param profile: Profile to build on (Optional)
    :return: True(Succeed) or False
    '''
    conn = _auth(profile)
    return conn.delete_port(port)


def list_networks(profile=None):
    '''
    Fetches a list of all networks for a tenant

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.list_networks
        salt '*' neutron.list_networks profile=openstack1

    :param profile: Profile to build on (Optional)
    :return: List of network
    '''
    conn = _auth(profile)
    return conn.list_networks()


def show_network(network, profile=None):
    '''
    Fetches information of a certain network

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.show_network network-name
        salt '*' neutron.show_network network-name profile=openstack1

    :param network: ID or name of network to look up
    :param profile: Profile to build on (Optional)
    :return: Network information
    '''
    conn = _auth(profile)
    return conn.show_network(network)


def create_network(name, router_ext=None, admin_state_up=True, network_type=None, physical_network=None, segmentation_id=None, shared=None, profile=None):
    '''
    Creates a new network

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.create_network network-name
        salt '*' neutron.create_network network-name profile=openstack1

    :param name: Name of network to create
    :param admin_state_up: should the state of the network be up?
            default: True (Optional)
    :param router_ext: True then if create the external network (Optional)
    :param network_type: the Type of network that the provider is such as GRE, VXLAN, VLAN, FLAT, or LOCAL (Optional)
    :param physical_network: the name of the physical network as neutron knows it (Optional)
    :param segmentation_id: the vlan id or GRE id (Optional)
    :param shared: is the network shared or not (Optional)
    :param profile: Profile to build on (Optional)
    :return: Created network information
    '''
    conn = _auth(profile)
    return conn.create_network(name, admin_state_up, router_ext, network_type, physical_network, segmentation_id, shared)


def update_network(network, name, profile=None):
    '''
    Updates a network

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.update_network network-name new-network-name

    :param network: ID or name of network to update
    :param name: Name of this network
    :param profile: Profile to build on (Optional)
    :return: Value of updated network information
    '''
    conn = _auth(profile)
    return conn.update_network(network, name)


def delete_network(network, profile=None):
    '''
    Deletes the specified network

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.delete_network network-name
        salt '*' neutron.delete_network network-name profile=openstack1

    :param network: ID or name of network to delete
    :param profile: Profile to build on (Optional)
    :return: True(Succeed) or False
    '''
    conn = _auth(profile)
    return conn.delete_network(network)


def list_subnets(profile=None):
    '''
    Fetches a list of all networks for a tenant

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.list_subnets
        salt '*' neutron.list_subnets profile=openstack1

    :param profile: Profile to build on (Optional)
    :return: List of subnet
    '''
    conn = _auth(profile)
    return conn.list_subnets()


def show_subnet(subnet, profile=None):
    '''
    Fetches information of a certain subnet

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.show_subnet subnet-name

    :param subnet: ID or name of subnet to look up
    :param profile: Profile to build on (Optional)
    :return: Subnet information
    '''
    conn = _auth(profile)
    return conn.show_subnet(subnet)


def create_subnet(network, cidr, name=None,
                  ip_version=4, profile=None):
    '''
    Creates a new subnet

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.create_subnet network-name 192.168.1.0/24

    :param network: Network ID or name this subnet belongs to
    :param cidr: CIDR of subnet to create (Ex. '192.168.1.0/24')
    :param name: Name of the subnet to create (Optional)
    :param ip_version: Version to use, default is 4(IPv4) (Optional)
    :param profile: Profile to build on (Optional)
    :return: Created subnet information
    '''
    conn = _auth(profile)
    return conn.create_subnet(network, cidr, name, ip_version)


def update_subnet(subnet, name, profile=None):
    '''
    Updates a subnet

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.update_subnet subnet-name new-subnet-name

    :param subnet: ID or name of subnet to update
    :param name: Name of this subnet
    :param profile: Profile to build on (Optional)
    :return: Value of updated subnet information
    '''
    conn = _auth(profile)
    return conn.update_subnet(subnet, name)


def delete_subnet(subnet, profile=None):
    '''
    Deletes the specified subnet

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.delete_subnet subnet-name
        salt '*' neutron.delete_subnet subnet-name profile=openstack1

    :param subnet: ID or name of subnet to delete
    :param profile: Profile to build on (Optional)
    :return: True(Succeed) or False
    '''
    conn = _auth(profile)
    return conn.delete_subnet(subnet)


def list_routers(profile=None):
    '''
    Fetches a list of all routers for a tenant

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.list_routers
        salt '*' neutron.list_routers profile=openstack1

    :param profile: Profile to build on (Optional)
    :return: List of router
    '''
    conn = _auth(profile)
    return conn.list_routers()


def show_router(router, profile=None):
    '''
    Fetches information of a certain router

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.show_router router-name

    :param router: ID or name of router to look up
    :param profile: Profile to build on (Optional)
    :return: Router information
    '''
    conn = _auth(profile)
    return conn.show_router(router)


def create_router(name, ext_network=None,
                  admin_state_up=True, profile=None):
    '''
    Creates a new router

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.create_router new-router-name

    :param name: Name of router to create (must be first)
    :param ext_network: ID or name of the external for the gateway (Optional)
    :param admin_state_up: Set admin state up to true or false,
            default:true (Optional)
    :param profile: Profile to build on (Optional)
    :return: Created router information
    '''
    conn = _auth(profile)
    return conn.create_router(name, ext_network, admin_state_up)


def update_router(router,
                  name=None,
                  admin_state_up=None,
                  profile=None,
                  **kwargs):
    '''
    Updates a router

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.update_router router_id name=new-router-name
                admin_state_up=True

    :param router: ID or name of router to update
    :param name: Name of this router
    :param ext_network: ID or name of the external for the gateway (Optional)
    :param admin_state_up: Set admin state up to true or false,
            default: true (Optional)
    :param profile: Profile to build on (Optional)
    :param kwargs:
    :return: Value of updated router information
    '''
    conn = _auth(profile)
    return conn.update_router(router, name, admin_state_up, **kwargs)


def delete_router(router, profile=None):
    '''
    Delete the specified router

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.delete_router router-name

    :param router: ID or name of router to delete
    :param profile: Profile to build on (Optional)
    :return: True(Succeed) or False
    '''
    conn = _auth(profile)
    return conn.delete_router(router)


def add_interface_router(router, subnet, profile=None):
    '''
    Adds an internal network interface to the specified router

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.add_interface_router router-name subnet-name

    :param router: ID or name of the router
    :param subnet: ID or name of the subnet
    :param profile: Profile to build on (Optional)
    :return: Added interface information
    '''
    conn = _auth(profile)
    return conn.add_interface_router(router, subnet)


def remove_interface_router(router, subnet, profile=None):
    '''
    Removes an internal network interface from the specified router

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.remove_interface_router router-name subnet-name

    :param router: ID or name of the router
    :param subnet: ID or name of the subnet
    :param profile: Profile to build on (Optional)
    :return: True(Succeed) or False
    '''
    conn = _auth(profile)
    return conn.remove_interface_router(router, subnet)


def add_gateway_router(router, ext_network, profile=None):
    '''
    Adds an external network gateway to the specified router

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.add_gateway_router router-name ext-network-name

    :param router: ID or name of the router
    :param ext_network: ID or name of the external network the gateway
    :param profile: Profile to build on (Optional)
    :return: Added Gateway router information
    '''
    conn = _auth(profile)
    return conn.add_gateway_router(router, ext_network)


def remove_gateway_router(router, profile=None):
    '''
    Removes an external network gateway from the specified router

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.remove_gateway_router router-name

    :param router: ID or name of router
    :param profile: Profile to build on (Optional)
    :return: True(Succeed) or False
    '''
    conn = _auth(profile)
    return conn.remove_gateway_router(router)


def list_floatingips(profile=None):
    '''
    Fetch a list of all floatingIPs for a tenant

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.list_floatingips
        salt '*' neutron.list_floatingips profile=openstack1

    :param profile: Profile to build on (Optional)
    :return: List of floatingIP
    '''
    conn = _auth(profile)
    return conn.list_floatingips()


def show_floatingip(floatingip_id, profile=None):
    '''
    Fetches information of a certain floatingIP

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.show_floatingip floatingip-id

    :param floatingip_id: ID of floatingIP to look up
    :param profile: Profile to build on (Optional)
    :return: Floating IP information
    '''
    conn = _auth(profile)
    return conn.show_floatingip(floatingip_id)


def create_floatingip(floating_network, port=None, profile=None):
    '''
    Creates a new floatingIP

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.create_floatingip network-name port-name

    :param floating_network: Network name or ID to allocate floatingIP from
    :param port: Of the port to be associated with the floatingIP (Optional)
    :param profile: Profile to build on (Optional)
    :return: Created floatingIP information
    '''
    conn = _auth(profile)
    return conn.create_floatingip(floating_network, port)


def update_floatingip(floatingip_id, port, profile=None):
    '''
    Updates a floatingIP

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.update_floatingip network-name port-name

    :param floatingip_id: ID of floatingIP
    :param port: ID or name of port
    :param profile: Profile to build on (Optional)
    :return: Value of updated floating IP information
    '''
    conn = _auth(profile)
    return conn.update_floatingip(floatingip_id, port)


def delete_floatingip(floatingip_id, profile=None):
    '''
    Deletes the specified floating IP

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.delete_floatingip floatingip-id

    :param floatingip_id: ID of floatingIP to delete
    :param profile: Profile to build on (Optional)
    :return: True(Succeed) or False
    '''
    conn = _auth(profile)
    return conn.delete_floatingip(floatingip_id)


def list_security_groups(profile=None):
    '''
    Fetches a list of all security groups for a tenant

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.list_security_groups
        salt '*' neutron.list_security_groups profile=openstack1

    :param profile: Profile to build on (Optional)
    :return: List of security group
    '''
    conn = _auth(profile)
    return conn.list_security_groups()


def show_security_group(security_group, profile=None):
    '''
    Fetches information of a certain security group

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.show_security_group security-group-name

    :param security_group: ID or name of security group to look up
    :param profile: Profile to build on (Optional)
    :return: Security group information
    '''
    conn = _auth(profile)
    return conn.show_security_group(security_group)


def create_security_group(name=None, description=None, profile=None):
    '''
    Creates a new security group

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.create_security_group security-group-name \
                description='Security group for servers'

    :param name: Name of security group (Optional)
    :param description: Description of security group (Optional)
    :param profile: Profile to build on (Optional)
    :return: Created security group information
    '''
    conn = _auth(profile)
    return conn.create_security_group(name, description)


def update_security_group(security_group, name=None, description=None,
                          profile=None):
    '''
    Updates a security group

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.update_security_group security-group-name \
                new-security-group-name

    :param security_group: ID or name of security group to update
    :param name: Name of this security group (Optional)
    :param description: Description of security group (Optional)
    :param profile: Profile to build on (Optional)
    :return: Value of updated security group information
    '''
    conn = _auth(profile)
    return conn.update_security_group(security_group, name, description)


def delete_security_group(security_group, profile=None):
    '''
    Deletes the specified security group

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.delete_security_group security-group-name

    :param security_group: ID or name of security group to delete
    :param profile: Profile to build on (Optional)
    :return: True(Succeed) or False
    '''
    conn = _auth(profile)
    return conn.delete_security_group(security_group)


def list_security_group_rules(profile=None):
    '''
    Fetches a list of all security group rules for a tenant

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.list_security_group_rules
        salt '*' neutron.list_security_group_rules profile=openstack1

    :param profile: Profile to build on (Optional)
    :return: List of security group rule
    '''
    conn = _auth(profile)
    return conn.list_security_group_rules()


def show_security_group_rule(security_group_rule_id, profile=None):
    '''
    Fetches information of a certain security group rule

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.show_security_group_rule security-group-rule-id

    :param security_group_rule_id: ID of security group rule to look up
    :param profile: Profile to build on (Optional)
    :return: Security group rule information
    '''
    conn = _auth(profile)
    return conn.show_security_group_rule(security_group_rule_id)


def create_security_group_rule(security_group,
                               remote_group_id=None,
                               direction='ingress',
                               protocol=None,
                               port_range_min=None,
                               port_range_max=None,
                               ethertype='IPv4',
                               profile=None):
    '''
    Creates a new security group rule

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.show_security_group_rule security-group-rule-id

    :param security_group: Security group name or ID to add rule
    :param remote_group_id: Remote security group name or ID to
            apply rule (Optional)
    :param direction: Direction of traffic: ingress/egress,
            default: ingress (Optional)
    :param protocol: Protocol of packet: null/icmp/tcp/udp,
            default: null (Optional)
    :param port_range_min: Starting port range (Optional)
    :param port_range_max: Ending port range (Optional)
    :param ethertype: IPv4/IPv6, default: IPv4 (Optional)
    :param profile: Profile to build on (Optional)
    :return: Created security group rule information
    '''
    conn = _auth(profile)
    return conn.create_security_group_rule(security_group,
                                           remote_group_id,
                                           direction,
                                           protocol,
                                           port_range_min,
                                           port_range_max,
                                           ethertype)


def delete_security_group_rule(security_group_rule_id, profile=None):
    '''
    Deletes the specified security group rule

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.delete_security_group_rule security-group-rule-id

    :param security_group_rule_id: ID of security group rule to delete
    :param profile: Profile to build on (Optional)
    :return: True(Succeed) or False
    '''
    conn = _auth(profile)
    return conn.delete_security_group_rule(security_group_rule_id)


def list_vpnservices(retrieve_all=True, profile=None, **kwargs):
    '''
    Fetches a list of all configured VPN services for a tenant

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.list_vpnservices

    :param retrieve_all: True or False, default: True (Optional)
    :param profile: Profile to build on (Optional)
    :return: List of VPN service
    '''
    conn = _auth(profile)
    return conn.list_vpnservices(retrieve_all, **kwargs)


def show_vpnservice(vpnservice, profile=None, **kwargs):
    '''
    Fetches information of a specific VPN service

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.show_vpnservice vpnservice-name

    :param vpnservice: ID or name of vpn service to look up
    :param profile: Profile to build on (Optional)
    :return: VPN service information
    '''
    conn = _auth(profile)
    return conn.show_vpnservice(vpnservice, **kwargs)


def create_vpnservice(subnet, router, name, admin_state_up=True, profile=None):
    '''
    Creates a new VPN service

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.create_vpnservice router-name name

    :param subnet: Subnet unique identifier for the VPN service deployment
    :param router: Router unique identifier for the VPN service
    :param name: Set a name for the VPN service
    :param admin_state_up: Set admin state up to true or false,
            default:True (Optional)
    :param profile: Profile to build on (Optional)
    :return: Created VPN service information
    '''
    conn = _auth(profile)
    return conn.create_vpnservice(subnet, router, name, admin_state_up)


def update_vpnservice(vpnservice, desc, profile=None):
    '''
    Updates a VPN service

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.update_vpnservice vpnservice-name desc='VPN Service1'

    :param vpnservice: ID or name of vpn service to update
    :param desc: Set a description for the VPN service
    :param profile: Profile to build on (Optional)
    :return: Value of updated VPN service information
    '''
    conn = _auth(profile)
    return conn.update_vpnservice(vpnservice, desc)


def delete_vpnservice(vpnservice, profile=None):
    '''
    Deletes the specified VPN service

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.delete_vpnservice vpnservice-name

    :param vpnservice: ID or name of vpn service to delete
    :param profile: Profile to build on (Optional)
    :return: True(Succeed) or False
    '''
    conn = _auth(profile)
    return conn.delete_vpnservice(vpnservice)


def list_ipsec_site_connections(profile=None):
    '''
    Fetches all configured IPsec Site Connections for a tenant

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.list_ipsec_site_connections
        salt '*' neutron.list_ipsec_site_connections profile=openstack1

    :param profile: Profile to build on (Optional)
    :return: List of IPSec site connection
    '''
    conn = _auth(profile)
    return conn.list_ipsec_site_connections()


def show_ipsec_site_connection(ipsec_site_connection, profile=None):
    '''
    Fetches information of a specific IPsecSiteConnection

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.show_ipsec_site_connection connection-name

    :param ipsec_site_connection: ID or name of ipsec site connection
            to look up
    :param profile: Profile to build on (Optional)
    :return: IPSec site connection information
    '''
    conn = _auth(profile)
    return conn.show_ipsec_site_connection(ipsec_site_connection)


def create_ipsec_site_connection(name,
                                 ipsecpolicy,
                                 ikepolicy,
                                 vpnservice,
                                 peer_cidrs,
                                 peer_address,
                                 peer_id,
                                 psk,
                                 admin_state_up=True,
                                 profile=None,
                                 **kwargs):
    '''
    Creates a new IPsecSiteConnection

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.show_ipsec_site_connection connection-name
                ipsec-policy-name ikepolicy-name vpnservice-name
                192.168.XXX.XXX/24 192.168.XXX.XXX 192.168.XXX.XXX secret

    :param name: Set friendly name for the connection
    :param ipsecpolicy: IPSec policy ID or name associated with this connection
    :param ikepolicy: IKE policy ID or name associated with this connection
    :param vpnservice: VPN service instance ID or name associated with
            this connection
    :param peer_cidrs: Remote subnet(s) in CIDR format
    :param peer_address: Peer gateway public IPv4/IPv6 address or FQDN
    :param peer_id: Peer router identity for authentication
            Can be IPv4/IPv6 address, e-mail address, key id, or FQDN
    :param psk: Pre-shared key string
    :param initiator: Initiator state in lowercase, default:bi-directional
    :param admin_state_up: Set admin state up to true or false,
            default: True (Optional)
    :param mtu: size for the connection, default:1500 (Optional)
    :param dpd_action: Dead Peer Detection attribute: hold/clear/disabled/
            restart/restart-by-peer (Optional)
    :param dpd_interval: Dead Peer Detection attribute (Optional)
    :param dpd_timeout: Dead Peer Detection attribute (Optional)
    :param profile: Profile to build on (Optional)
    :return: Created IPSec site connection information
    '''
    conn = _auth(profile)
    return conn.create_ipsec_site_connection(name,
                                             ipsecpolicy,
                                             ikepolicy,
                                             vpnservice,
                                             peer_cidrs,
                                             peer_address,
                                             peer_id,
                                             psk,
                                             admin_state_up,
                                             **kwargs)


def delete_ipsec_site_connection(ipsec_site_connection, profile=None):
    '''
    Deletes the specified IPsecSiteConnection

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.delete_ipsec_site_connection connection-name

    :param ipsec_site_connection: ID or name of ipsec site connection to delete
    :param profile: Profile to build on (Optional)
    :return: True(Succeed) or False
    '''
    conn = _auth(profile)
    return conn.delete_ipsec_site_connection(ipsec_site_connection)


def list_ikepolicies(profile=None):
    '''
    Fetches a list of all configured IKEPolicies for a tenant

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.list_ikepolicies
        salt '*' neutron.list_ikepolicies profile=openstack1

    :param profile: Profile to build on (Optional)
    :return: List of IKE policy
    '''
    conn = _auth(profile)
    return conn.list_ikepolicies()


def show_ikepolicy(ikepolicy, profile=None):
    '''
    Fetches information of a specific IKEPolicy

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.show_ikepolicy ikepolicy-name

    :param ikepolicy: ID or name of ikepolicy to look up
    :param profile: Profile to build on (Optional)
    :return: IKE policy information
    '''
    conn = _auth(profile)
    return conn.show_ikepolicy(ikepolicy)


def create_ikepolicy(name, profile=None, **kwargs):
    '''
    Creates a new IKEPolicy

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.create_ikepolicy ikepolicy-name
                phase1_negotiation_mode=main auth_algorithm=sha1
                encryption_algorithm=aes-128 pfs=group5

    :param name: Name of the IKE policy
    :param phase1_negotiation_mode: IKE Phase1 negotiation mode in lowercase,
            default: main (Optional)
    :param auth_algorithm: Authentication algorithm in lowercase,
            default: sha1 (Optional)
    :param encryption_algorithm: Encryption algorithm in lowercase.
            default:aes-128 (Optional)
    :param pfs: Prefect Forward Security in lowercase,
            default: group5 (Optional)
    :param units: IKE lifetime attribute. default: seconds (Optional)
    :param value: IKE lifetime attribute. default: 3600 (Optional)
    :param ike_version: IKE version in lowercase, default: v1 (Optional)
    :param profile: Profile to build on (Optional)
    :param kwargs:
    :return: Created IKE policy information
    '''
    conn = _auth(profile)
    return conn.create_ikepolicy(name, **kwargs)


def delete_ikepolicy(ikepolicy, profile=None):
    '''
    Deletes the specified IKEPolicy

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.delete_ikepolicy ikepolicy-name

    :param ikepolicy: ID or name of IKE policy to delete
    :param profile: Profile to build on (Optional)
    :return: True(Succeed) or False
    '''
    conn = _auth(profile)
    return conn.delete_ikepolicy(ikepolicy)


def list_ipsecpolicies(profile=None):
    '''
    Fetches a list of all configured IPsecPolicies for a tenant

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.list_ipsecpolicies ipsecpolicy-name
        salt '*' neutron.list_ipsecpolicies ipsecpolicy-name profile=openstack1

    :param profile: Profile to build on (Optional)
    :return: List of IPSec policy
    '''
    conn = _auth(profile)
    return conn.list_ipsecpolicies()


def show_ipsecpolicy(ipsecpolicy, profile=None):
    '''
    Fetches information of a specific IPsecPolicy

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.show_ipsecpolicy ipsecpolicy-name

    :param ipsecpolicy: ID or name of IPSec policy to look up
    :param profile: Profile to build on (Optional)
    :return: IPSec policy information
    '''
    conn = _auth(profile)
    return conn.show_ipsecpolicy(ipsecpolicy)


def create_ipsecpolicy(name, profile=None, **kwargs):
    '''
    Creates a new IPsecPolicy

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.create_ipsecpolicy ipsecpolicy-name
                transform_protocol=esp auth_algorithm=sha1
                encapsulation_mode=tunnel encryption_algorithm=aes-128

    :param name: Name of the IPSec policy
    :param transform_protocol: Transform protocol in lowercase,
            default: esp (Optional)
    :param auth_algorithm: Authentication algorithm in lowercase,
            default: sha1 (Optional)
    :param encapsulation_mode: Encapsulation mode in lowercase,
            default: tunnel (Optional)
    :param encryption_algorithm: Encryption algorithm in lowercase,
            default:aes-128 (Optional)
    :param pfs: Prefect Forward Security in lowercase,
            default: group5 (Optional)
    :param units: IPSec lifetime attribute. default: seconds (Optional)
    :param value: IPSec lifetime attribute. default: 3600 (Optional)
    :param profile: Profile to build on (Optional)
    :return: Created IPSec policy information
    '''
    conn = _auth(profile)
    return conn.create_ipsecpolicy(name, **kwargs)


def delete_ipsecpolicy(ipsecpolicy, profile=None):
    '''
    Deletes the specified IPsecPolicy

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.delete_ipsecpolicy ipsecpolicy-name

    :param ipsecpolicy: ID or name of IPSec policy to delete
    :param profile: Profile to build on (Optional)
    :return: True(Succeed) or False
    '''
    conn = _auth(profile)
    return conn.delete_ipsecpolicy(ipsecpolicy)


def list_firewall_rules(profile=None):
    '''
    Fetches a list of all firewall rules for a tenant
    CLI Example:

    .. code-block:: bash

        salt '*' neutron.list_firewall_rules

    :param profile: Profile to build on (Optional)

    :return: List of firewall rules
    '''
    conn = _auth(profile)
    return conn.list_firewall_rules()


def show_firewall_rule(firewall_rule, profile=None):
    '''
    Fetches information of a specific firewall rule

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.show_firewall_rule firewall-rule-name

    :param ipsecpolicy: ID or name of firewall rule to look up

    :param profile: Profile to build on (Optional)

    :return: firewall rule information
    '''
    conn = _auth(profile)
    return conn.show_firewall_rule(firewall_rule)


def create_firewall_rule(protocol, action, profile=None, **kwargs):
    '''
    Creates a new firewall rule

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.create_firewall_rule protocol action
                tenant_id=TENANT_ID name=NAME description=DESCRIPTION ip_version=IP_VERSION
                source_ip_address=SOURCE_IP_ADDRESS destination_ip_address=DESTINATION_IP_ADDRESS source_port=SOURCE_PORT
                destination_port=DESTINATION_PORT shared=SHARED enabled=ENABLED

    :param protocol: Protocol for the firewall rule, choose "tcp","udp","icmp" or "None".
    :param action: Action for the firewall rule, choose "allow" or "deny".
    :param tenant_id: The owner tenant ID. (Optional)
    :param name: Name for the firewall rule. (Optional)
    :param description: Description for the firewall rule. (Optional)
    :param ip_version: IP protocol version, default: 4. (Optional)
    :param source_ip_address: Source IP address or subnet. (Optional)
    :param destination_ip_address: Destination IP address or subnet. (Optional)
    :param source_port: Source port (integer in [1, 65535] or range in a:b). (Optional)
    :param destination_port: Destination port (integer in [1, 65535] or range in a:b). (Optional)
    :param shared: Set shared to True, default: False. (Optional)
    :param enabled: To enable this rule, default: True. (Optional)
    '''
    conn = _auth(profile)
    return conn.create_firewall_rule(protocol, action, **kwargs)


def delete_firewall_rule(firewall_rule, profile=None):
    '''
    Deletes the specified firewall_rule

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.delete_firewall_rule firewall-rule

    :param firewall_rule: ID or name of firewall rule to delete
    :param profile: Profile to build on (Optional)
    :return: True(Succeed) or False
    '''
    conn = _auth(profile)
    return conn.delete_firewall_rule(firewall_rule)


def update_firewall_rule(firewall_rule,
                         protocol=None,
                         action=None,
                         name=None,
                         description=None,
                         ip_version=None,
                         source_ip_address=None,
                         destination_ip_address=None,
                         source_port=None,
                         destination_port=None,
                         shared=None,
                         enabled=None,
                         profile=None):
    '''
    Update a firewall rule

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.update_firewall_rule firewall_rule protocol=PROTOCOL action=ACTION
                name=NAME description=DESCRIPTION ip_version=IP_VERSION
                source_ip_address=SOURCE_IP_ADDRESS destination_ip_address=DESTINATION_IP_ADDRESS
                source_port=SOURCE_PORT destination_port=DESTINATION_PORT shared=SHARED enabled=ENABLED

    :param firewall_rule: ID or name of firewall rule to update.
    :param protocol: Protocol for the firewall rule, choose "tcp","udp","icmp" or "None". (Optional)
    :param action: Action for the firewall rule, choose "allow" or "deny". (Optional)
    :param name: Name for the firewall rule. (Optional)
    :param description: Description for the firewall rule. (Optional)
    :param ip_version: IP protocol version, default: 4. (Optional)
    :param source_ip_address: Source IP address or subnet. (Optional)
    :param destination_ip_address: Destination IP address or subnet. (Optional)
    :param source_port: Source port (integer in [1, 65535] or range in a:b). (Optional)
    :param destination_port: Destination port (integer in [1, 65535] or range in a:b). (Optional)
    :param shared: Set shared to True, default: False. (Optional)
    :param enabled: To enable this rule, default: True. (Optional)
    :param profile: Profile to build on (Optional)
    '''
    conn = _auth(profile)
    return conn.update_firewall_rule(firewall_rule, protocol, action, name, description, ip_version,
                                     source_ip_address, destination_ip_address, source_port, destination_port,
                                     shared, enabled)


def list_firewalls(profile=None):
    '''
    Fetches a list of all firewalls for a tenant
    CLI Example:

    .. code-block:: bash

        salt '*' neutron.list_firewalls
    :param profile: Profile to build on (Optional)
    :return: List of firewalls
    '''
    conn = _auth(profile)
    return conn.list_firewalls()


def show_firewall(firewall, profile=None):
    '''
    Fetches information of a specific firewall rule

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.show_firewall firewall

    :param firewall: ID or name of firewall to look up
    :param profile: Profile to build on (Optional)
    :return: firewall information
    '''
    conn = _auth(profile)
    return conn.show_firewall(firewall)


def list_l3_agent_hosting_routers(router, profile=None):
    '''
    List L3 agents hosting a router.

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.list_l3_agent_hosting_routers router

    :param router:router name or ID to query.
    :param profile: Profile to build on (Optional)
    :return: L3 agents message.
    '''
    conn = _auth(profile)
    return conn.list_l3_agent_hosting_routers(router)


def list_agents(profile=None):
    '''
    List agents.

    CLI Example:

    .. code-block:: bash

        salt '*' neutron.list_agents

    :param profile: Profile to build on (Optional)
    :return: agents message.
    '''
    conn = _auth(profile)
    return conn.list_agents()


# The following is a list of functions that need to be incorporated in the
# neutron module. This list should be updated as functions are added.
#
# update_ipsec_site_connection
#                       Updates an IPsecSiteConnection.
# update_ikepolicy      Updates an IKEPolicy
# update_ipsecpolicy    Updates an IPsecPolicy
# list_vips             Fetches a list of all load balancer vips for a tenant.
# show_vip              Fetches information of a certain load balancer vip.
# create_vip            Creates a new load balancer vip.
# update_vip            Updates a load balancer vip.
# delete_vip            Deletes the specified load balancer vip.
# list_pools            Fetches a list of all load balancer pools for a tenant.
# show_pool             Fetches information of a certain load balancer pool.
# create_pool           Creates a new load balancer pool.
# update_pool           Updates a load balancer pool.
# delete_pool           Deletes the specified load balancer pool.
# retrieve_pool_stats   Retrieves stats for a certain load balancer pool.
# list_members          Fetches a list of all load balancer members for
#                       a tenant.
# show_member           Fetches information of a certain load balancer member.
# create_member         Creates a new load balancer member.
# update_member         Updates a load balancer member.
# delete_member         Deletes the specified load balancer member.
# list_health_monitors  Fetches a list of all load balancer health monitors for
#                       a tenant.
# show_health_monitor   Fetches information of a certain load balancer
#                       health monitor.
# create_health_monitor
#                       Creates a new load balancer health monitor.
# update_health_monitor
#                       Updates a load balancer health monitor.
# delete_health_monitor
#                       Deletes the specified load balancer health monitor.
# associate_health_monitor
#                       Associate  specified load balancer health monitor
#                       and pool.
# disassociate_health_monitor
#                       Disassociate specified load balancer health monitor
#                       and pool.
# create_qos_queue      Creates a new queue.
# list_qos_queues       Fetches a list of all queues for a tenant.
# show_qos_queue        Fetches information of a certain queue.
# delete_qos_queue      Deletes the specified queue.
# list_agents           Fetches agents.
# show_agent            Fetches information of a certain agent.
# update_agent          Updates an agent.
# delete_agent          Deletes the specified agent.
# list_network_gateways
#                       Retrieve network gateways.
# show_network_gateway  Fetch a network gateway.
# create_network_gateway
#                       Create a new network gateway.
# update_network_gateway
#                       Update a network gateway.
# delete_network_gateway
#                       Delete the specified network gateway.
# connect_network_gateway
#                       Connect a network gateway to the specified network.
# disconnect_network_gateway
#                       Disconnect a network from the specified gateway.
# list_gateway_devices  Retrieve gateway devices.
# show_gateway_device   Fetch a gateway device.
# create_gateway_device
#                       Create a new gateway device.
# update_gateway_device
#                       Updates a new gateway device.
# delete_gateway_device
#                       Delete the specified gateway device.
# list_dhcp_agent_hosting_networks
#                       Fetches a list of dhcp agents hosting a network.
# list_networks_on_dhcp_agent
#                       Fetches a list of dhcp agents hosting a network.
# add_network_to_dhcp_agent
#                       Adds a network to dhcp agent.
# remove_network_from_dhcp_agent
#                       Remove a network from dhcp agent.
# list_l3_agent_hosting_routers
#                       Fetches a list of L3 agents hosting a router.
# list_routers_on_l3_agent
#                       Fetches a list of L3 agents hosting a router.
# add_router_to_l3_agent
#                       Adds a router to L3 agent.
# list_firewall_rules   Fetches a list of all firewall rules for a tenant.
# show_firewall_rule    Fetches information of a certain firewall rule.
# create_firewall_rule  Creates a new firewall rule.
# update_firewall_rule  Updates a firewall rule.
# delete_firewall_rule  Deletes the specified firewall rule.
# list_firewall_policies
#                       Fetches a list of all firewall policies for a tenant.
# show_firewall_policy  Fetches information of a certain firewall policy.
# create_firewall_policy
#                       Creates a new firewall policy.
# update_firewall_policy
#                       Updates a firewall policy.
# delete_firewall_policy
#                       Deletes the specified firewall policy.
# firewall_policy_insert_rule
#                       Inserts specified rule into firewall policy.
# firewall_policy_remove_rule
#                       Removes specified rule from firewall policy.
# list_firewalls        Fetches a list of all firewals for a tenant.
# show_firewall         Fetches information of a certain firewall.
# create_firewall       Creates a new firewall.
# update_firewall       Updates a firewall.
# delete_firewall       Deletes the specified firewall.
# remove_router_from_l3_agent
#                       Remove a router from l3 agent.
# get_lbaas_agent_hosting_pool
#                       Fetches a loadbalancer agent hosting a pool.
# list_pools_on_lbaas_agent
#                       Fetches a list of pools hosted by
#                       the loadbalancer agent.
# list_service_providers
#                       Fetches service providers.
# list_credentials      Fetch a list of all credentials for a tenant.
# show_credential       Fetch a credential.
# create_credential     Create a new credential.
# update_credential     Update a credential.
# delete_credential     Delete the specified credential.
# list_network_profile_bindings
#                       Fetch a list of all tenants associated for
#                       a network profile.
# list_network_profiles
#                       Fetch a list of all network profiles for a tenant.
# show_network_profile  Fetch a network profile.
# create_network_profile
#                       Create a network profile.
# update_network_profile
#                       Update a network profile.
# delete_network_profile
#                       Delete the network profile.
# list_policy_profile_bindings
#                       Fetch a list of all tenants associated for
#                       a policy profile.
# list_policy_profiles  Fetch a list of all network profiles for a tenant.
# show_policy_profile   Fetch a network profile.
# update_policy_profile
#                       Update a policy profile.
# create_metering_label
#                       Creates a metering label.
# delete_metering_label
#                       Deletes the specified metering label.
# list_metering_labels  Fetches a list of all metering labels for a tenant.
# show_metering_label   Fetches information of a certain metering label.
# create_metering_label_rule
#                       Creates a metering label rule.
# delete_metering_label_rule
#                       Deletes the specified metering label rule.
# list_metering_label_rules
#                       Fetches a list of all metering label rules for a label.
# show_metering_label_rule
#                       Fetches information of a certain metering label rule.
# list_net_partitions   Fetch a list of all network partitions for a tenant.
# show_net_partition    etch a network partition.
# create_net_partition  Create a network partition.
# delete_net_partition  Delete the network partition.
# create_packet_filter  Create a new packet filter.
# update_packet_filter  Update a packet filter.
# list_packet_filters   Fetch a list of all packet filters for a tenant.
# show_packet_filter    Fetch information of a certain packet filter.
# delete_packet_filter  Delete the specified packet filter.



# -*- coding: utf-8 -*-
'''
Return data to an etcd server or cluster

:depends: - python-etcd

In order to return to an etcd server, a profile should be created in the master
configuration file:

.. code-block:: yaml

    my_etcd_config:
      etcd.host: 127.0.0.1
      etcd.port: 4001

It is technically possible to configure etcd without using a profile, but this
is not considered to be a best practice, especially when multiple etcd servers
or clusters are available.

.. code-block:: yaml

    etcd.host: 127.0.0.1
    etcd.port: 4001

Additionally, two more options must be specified in the top-level configuration
in order to use the etcd returner:

.. code-block:: yaml

    etcd.returner: my_etcd_config
    etcd.returner_root: /salt/return

The ``etcd.returner`` option specifies which configuration profile to use. The
``etcd.returner_root`` option specifies the path inside etcd to use as the root
of the returner system.

Once the etcd options are configured, the returner may be used:

CLI Example:

    salt '*' test.ping --return etcd

A username and password can be set:

.. code-block:: yaml

    etcd.username: larry  # Optional; requires etcd.password to be set
    etcd.password: 123pass  # Optional; requires etcd.username to be set

You can also set a TTL (time to live) value for the returner:

.. code-block:: yaml

    etcd.ttl: 5

Authentication with username and password, and ttl, currently requires the
``master`` branch of ``python-etcd``.

You may also specify different roles for read and write operations. First,
create the profiles as specified above. Then add:

.. code-block:: yaml

    etcd.returner_read_profile: my_etcd_read
    etcd.returner_write_profile: my_etcd_write
'''
from __future__ import absolute_import

# Import python libs
import json
import logging

# Import salt libs
try:
    import salt.utils.etcd_util
    HAS_LIBS = True
except ImportError:
    HAS_LIBS = False

import salt.utils
import salt.utils.jid

log = logging.getLogger(__name__)

# Define the module's virtual name
__virtualname__ = 'etcd'


def __virtual__():
    '''
    Only return if python-etcd is installed
    '''
    if HAS_LIBS:
        return __virtualname__

    return False, 'Could not import etcd returner; python-etcd is not installed.'


def _get_conn(opts, profile=None):
    '''
    Establish a connection to etcd
    '''
    if profile is None:
        profile = opts.get('etcd.returner')
    path = opts.get('etcd.returner_root', '/salt/return')
    return salt.utils.etcd_util.get_conn(opts, profile), path


def returner(ret):
    '''
    Return data to an etcd server or cluster
    '''
    write_profile = __opts__.get('etcd.returner_write_profile')
    if write_profile:
        ttl = __opts__.get(write_profile, {}).get('etcd.ttl')
    else:
        ttl = __opts__.get('etcd.ttl')

    client, path = _get_conn(__opts__, write_profile)
    # Make a note of this minion for the external job cache
    client.set(
        '/'.join((path, 'minions', ret['id'])),
        ret['jid'],
        ttl=ttl,
    )

    for field in ret:
        # Not using os.path.join because we're not dealing with file paths
        dest = '/'.join((
            path,
            'jobs',
            ret['jid'],
            ret['id'],
            field
        ))
        client.set(dest, json.dumps(ret[field]), ttl=ttl)


def save_load(jid, load, minions=None):
    '''
    Save the load to the specified jid
    '''
    write_profile = __opts__.get('etcd.returner_write_profile')
    client, path = _get_conn(__opts__, write_profile)
    if write_profile:
        ttl = __opts__.get(write_profile, {}).get('etcd.ttl')
    else:
        ttl = __opts__.get('etcd.ttl')
    client.set(
        '/'.join((path, 'jobs', jid, '.load.p')),
        json.dumps(load),
        ttl=ttl,
    )


def save_minions(jid, minions, syndic_id=None):  # pylint: disable=unused-argument
    '''
    Included for API consistency
    '''
    pass


def get_load(jid):
    '''
    Return the load data that marks a specified jid
    '''
    read_profile = __opts__.get('etcd.returner_read_profile')
    client, path = _get_conn(__opts__, read_profile)
    return json.loads(client.get('/'.join((path, 'jobs', jid, '.load.p'))))


def get_jid(jid):
    '''
    Return the information returned when the specified job id was executed
    '''
    client, path = _get_conn(__opts__)
    jid_path = '/'.join((path, 'jobs', jid))
    return client.tree(jid_path)


def get_fun():
    '''
    Return a dict of the last function called for all minions
    '''
    ret = {}
    client, path = _get_conn(__opts__)
    items = client.get('/'.join((path, 'minions')))
    for item in items.children:
        comps = str(item.key).split('/')
        ret[comps[-1]] = item.value
    return ret


def get_jids():
    '''
    Return a list of all job ids
    '''
    ret = {}
    client, path = _get_conn(__opts__)
    items = client.get('/'.join((path, 'jobs')))
    for item in items.children:
        if item.dir is True:
            jid = str(item.key).split('/')[-1]
            load = client.get('/'.join((item.key, '.load.p'))).value
            ret[jid] = salt.utils.jid.format_jid_instance(jid, json.loads(load))
    return ret


def get_minions():
    '''
    Return a list of minions
    '''
    ret = []
    client, path = _get_conn(__opts__)
    items = client.get('/'.join((path, 'minions')))
    for item in items.children:
        comps = str(item.key).split('/')
        ret.append(comps[-1])
    return ret


def prep_jid(nocache=False, passed_jid=None):  # pylint: disable=unused-argument
    '''
    Do any work necessary to prepare a JID, including sending a custom id
    '''
    return passed_jid if passed_jid is not None else salt.utils.jid.gen_jid()



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Nicole Thomas <nicole@saltstack.com>`
'''

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import MagicMock, NO_MOCK, NO_MOCK_REASON, patch
from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../../')

# Import Salt Libs
from salt.cloud.clouds import opennebula
from salt.exceptions import SaltCloudSystemExit, SaltCloudNotFound

# Import Third Party Libs
try:
    from lxml import etree  # pylint: disable=W0611
    HAS_XML_LIBS = True
except ImportError:
    HAS_XML_LIBS = False

# Global Variables
opennebula.__active_provider_name__ = ''
opennebula.__opts__ = {}
opennebula.__utils__ = {}
opennebula.__utils__['cloud.cache_node'] = MagicMock()
VM_NAME = 'my-vm'


@skipIf(NO_MOCK, NO_MOCK_REASON)
@patch('salt.cloud.clouds.opennebula.__virtual__', MagicMock(return_value='opennebula'))
class OpenNebulaTestCase(TestCase):
    '''
    Unit TestCase for salt.cloud.clouds.opennebula module.
    '''

    def test_avail_images_action(self):
        '''
        Tests that a SaltCloudSystemExit error is raised when trying to call
        avail_images with --action or -a.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.avail_images, 'action')

    def test_avail_locations_action(self):
        '''
        Tests that a SaltCloudSystemExit is raised when trying to call avail_locations
        with --action or -a.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.avail_locations, 'action')

    def test_avail_sizes_action(self):
        '''
        Tests that a SaltCloudSystemExit is raised when trying to call avail_sizes
        with --action or -a.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.avail_sizes, 'action')

    def test_avail_sizes(self):
        '''
        Tests that avail_sizes returns an empty dictionary.
        '''
        self.assertEqual(opennebula.avail_sizes(call='foo'), {})

    def test_list_clusters_action(self):
        '''
        Tests that a SaltCloudSystemExit is raised when trying to call list_clusters
        with --action or -a.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.list_clusters, 'action')

    def test_list_datastores_action(self):
        '''
        Tests that a SaltCloudSystemExit is raised when trying to call list_datastores
        with --action or -a.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.list_datastores, 'action')

    def test_list_hosts_action(self):
        '''
        Tests that a SaltCloudSystemExit is raised when trying to call list_datastores
        with --action or -a.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.list_hosts, 'action')

    def test_list_nodes_action(self):
        '''
        Tests that a SaltCloudSystemExit is raised when trying to call list_nodes
        with --action or -a.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.list_nodes, 'action')

    def test_list_nodes_full_action(self):
        '''
        Tests that a SaltCloudSystemExit is raised when trying to call list_nodes_full
        with --action or -a.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.list_nodes_full, 'action')

    def test_list_nodes_select_action(self):
        '''
        Tests that a SaltCloudSystemExit is raised when trying to call list_nodes_full
        with --action or -a.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.list_nodes_select, 'action')

    def test_list_security_groups_action(self):
        '''
        Tests that a SaltCloudSystemExit is raised when trying to call
        list_security_groups with --action or -a.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.list_security_groups, 'action')

    def test_list_templates_action(self):
        '''
        Tests that a SaltCloudSystemExit is raised when trying to call list_templates
        with --action or -a.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.list_templates, 'action')

    def test_list_vns_action(self):
        '''
        Tests that a SaltCloudSystemExit is raised when trying to call list_vns
        with --action or -a.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.list_vns, 'action')

    def test_reboot_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when trying to call reboot
        with anything other that --action or -a.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.reboot, 'my-vm', 'foo')

    def test_start_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when trying to call start
        with anything other that --action or -a.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.start, 'my-vm', 'foo')

    def test_stop_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when trying to call stop
        with anything other that --action or -a.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.stop, 'my-vm', 'foo')

    def test_get_cluster_id_action(self):
        '''
        Tests that a SaltCloudSystemExit is raised when trying to call
        get_cluster_id with --action or -a.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_cluster_id,
                          call='action')

    def test_get_cluster_id_no_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when no name is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_cluster_id,
                          None,
                          call='foo')

    @patch('salt.cloud.clouds.opennebula.list_clusters',
           MagicMock(return_value={'foo': {'id': 'bar'}}))
    def test_get_cluster_id_not_found(self):
        '''
        Tests that a SaltCloudSystemExit is raised when no name is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_cluster_id,
                          kwargs={'name': 'test'},
                          call='function')

    @patch('salt.cloud.clouds.opennebula.list_clusters',
           MagicMock(return_value={'test-cluster': {'id': '100'}}))
    def test_get_cluster_id_success(self):
        '''
        Tests that the function returns successfully.
        '''
        mock_id = '100'
        mock_kwargs = {'name': 'test-cluster'}
        self.assertEqual(opennebula.get_cluster_id(mock_kwargs, 'foo'),
                         mock_id)

    def test_get_datastore_id_action(self):
        '''
        Tests that a SaltCloudSystemExit is raised when trying to call
        get_datastore_id with --action or -a.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_datastore_id,
                          call='action')

    def test_get_datastore_id_no_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when no name is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_datastore_id,
                          None,
                          call='foo')

    @patch('salt.cloud.clouds.opennebula.list_datastores',
           MagicMock(return_value={'test-datastore': {'id': '100'}}))
    def test_get_datastore_id_not_found(self):
        '''
        Tests that a SaltCloudSystemExit is raised when no name is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_datastore_id,
                          kwargs={'name': 'test'},
                          call='function')

    @patch('salt.cloud.clouds.opennebula.list_datastores',
           MagicMock(return_value={'test-datastore': {'id': '100'}}))
    def test_get_datastore_id_success(self):
        '''
        Tests that the function returns successfully.
        '''
        mock_id = '100'
        mock_kwargs = {'name': 'test-datastore'}
        self.assertEqual(opennebula.get_datastore_id(mock_kwargs, 'foo'),
                         mock_id)

    def test_get_host_id_action(self):
        '''
        Tests that a SaltCloudSystemExit is raised when trying to call
        get_host_id with --action or -a.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_host_id,
                          call='action')

    def test_get_host_id_no_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when no name is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_host_id,
                          None,
                          call='foo')

    @patch('salt.cloud.clouds.opennebula.avail_locations',
           MagicMock(return_value={'test-host': {'id': '100'}}))
    def test_get_host_id_not_found(self):
        '''
        Tests that a SaltCloudSystemExit is raised when no name is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_host_id,
                          kwargs={'name': 'test'},
                          call='function')

    @patch('salt.cloud.clouds.opennebula.avail_locations',
           MagicMock(return_value={'test-host': {'id': '100'}}))
    def test_get_host_id_success(self):
        '''
        Tests that the function returns successfully.
        '''
        mock_id = '100'
        mock_kwargs = {'name': 'test-host'}
        self.assertEqual(opennebula.get_host_id(mock_kwargs, 'foo'),
                         mock_id)

    @patch('salt.cloud.clouds.opennebula.avail_images', MagicMock(return_value={}))
    @patch('salt.config.get_cloud_config_value', MagicMock(return_value='foo'))
    def test_get_image_not_found(self):
        '''
        Tests that a SaltCloudNotFound is raised when the image doesn't exist.
        '''
        self.assertRaises(SaltCloudNotFound, opennebula.get_image, 'my-vm')

    @patch('salt.cloud.clouds.opennebula.avail_images',
           MagicMock(return_value={'my-vm': {'name': 'my-vm', 'id': 0}}))
    @patch('salt.config.get_cloud_config_value', MagicMock(return_value='my-vm'))
    def test_get_image_success(self):
        '''
        Tests that the image is returned successfully.
        '''
        self.assertEqual(opennebula.get_image('my-vm'), 0)

    def test_get_image_id_action(self):
        '''
        Tests that a SaltCloudSystemExit is raised when trying to call
        get_image_id with --action or -a.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_image_id,
                          call='action')

    def test_get_image_id_no_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when no name is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_image_id,
                          None,
                          call='foo')

    @patch('salt.cloud.clouds.opennebula.avail_images',
           MagicMock(return_value={'test-image': {'id': '100'}}))
    def test_get_image_id_not_found(self):
        '''
        Tests that a SaltCloudSystemExit is raised when no name is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_image_id,
                          kwargs={'name': 'test'},
                          call='function')

    @patch('salt.cloud.clouds.opennebula.avail_images',
           MagicMock(return_value={'test-image': {'id': '100'}}))
    def test_get_image_id_success(self):
        '''
        Tests that the function returns successfully.
        '''
        mock_id = '100'
        mock_kwargs = {'name': 'test-image'}
        self.assertEqual(opennebula.get_image_id(mock_kwargs, 'foo'),
                         mock_id)

    @patch('salt.cloud.clouds.opennebula.avail_locations', MagicMock(return_value={}))
    @patch('salt.config.get_cloud_config_value', MagicMock(return_value='foo'))
    def test_get_location_not_found(self):
        '''
        Tests that a SaltCloudNotFound is raised when the location doesn't exist.
        '''
        self.assertRaises(SaltCloudNotFound, opennebula.get_location, 'my-vm')

    @patch('salt.cloud.clouds.opennebula.avail_locations',
           MagicMock(return_value={'my-host': {'name': 'my-host', 'id': 0}}))
    @patch('salt.config.get_cloud_config_value', MagicMock(return_value='my-host'))
    def test_get_location_success(self):
        '''
        Tests that the image is returned successfully.
        '''
        self.assertEqual(opennebula.get_location('my-host'), 0)

    def test_get_secgroup_id_action(self):
        '''
        Tests that a SaltCloudSystemExit is raised when trying to call
        get_host_id with --action or -a.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_secgroup_id,
                          call='action')

    def test_get_secgroup_id_no_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when no name is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_secgroup_id,
                          None,
                          call='foo')

    @patch('salt.cloud.clouds.opennebula.list_security_groups',
           MagicMock(return_value={'test-security-group': {'id': '100'}}))
    def test_get_secgroup_id_not_found(self):
        '''
        Tests that a SaltCloudSystemExit is raised when no name is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_secgroup_id,
                          kwargs={'name': 'test'},
                          call='function')

    @patch('salt.cloud.clouds.opennebula.list_security_groups',
           MagicMock(return_value={'test-secgroup': {'id': '100'}}))
    def test_get_secgroup_id_success(self):
        '''
        Tests that the function returns successfully.
        '''
        mock_id = '100'
        mock_kwargs = {'name': 'test-secgroup'}
        self.assertEqual(opennebula.get_secgroup_id(mock_kwargs, 'foo'),
                         mock_id)

    def test_get_template_id_action(self):
        '''
        Tests that a SaltCloudSystemExit is raised when trying to call
        get_template_id with --action or -a.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_template_id,
                          call='action')

    def test_get_template_id_no_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when no name is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_template_id,
                          None,
                          call='foo')

    @patch('salt.cloud.clouds.opennebula.list_templates',
           MagicMock(return_value={'test-template': {'id': '100'}}))
    def test_get_template_id_not_found(self):
        '''
        Tests that a SaltCloudSystemExit is raised when no name is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_template_id,
                          kwargs={'name': 'test'},
                          call='function')

    @patch('salt.cloud.clouds.opennebula.list_templates',
           MagicMock(return_value={'test-template': {'id': '100'}}))
    def test_get_template_id_success(self):
        '''
        Tests that the function returns successfully.
        '''
        mock_id = '100'
        mock_kwargs = {'name': 'test-template'}
        self.assertEqual(opennebula.get_template_id(mock_kwargs, 'foo'),
                         mock_id)

    def test_get_vm_id_action(self):
        '''
        Tests that a SaltCloudSystemExit is raised when trying to call
        get_vm_id with --action or -a.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_vm_id,
                          call='action')

    def test_get_vm_id_no_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when no name is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_vm_id,
                          None,
                          call='foo')

    @patch('salt.cloud.clouds.opennebula.list_nodes',
           MagicMock(return_value={'test-vm': {'id': '100'}}))
    def test_get_vm_id_not_found(self):
        '''
        Tests that a SaltCloudSystemExit is raised when no name is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_vm_id,
                          kwargs={'name': 'test'},
                          call='function')

    @patch('salt.cloud.clouds.opennebula.list_nodes',
           MagicMock(return_value={'test-vm': {'id': '100'}}))
    def test_get_vm_id_success(self):
        '''
        Tests that the function returns successfully.
        '''
        mock_id = '100'
        mock_kwargs = {'name': 'test-vm'}
        self.assertEqual(opennebula.get_vm_id(mock_kwargs, 'foo'),
                         mock_id)

    def test_get_vn_id_action(self):
        '''
        Tests that a SaltCloudSystemExit is raised when trying to call
        get_vn_id with --action or -a.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_vn_id,
                          call='action')

    def test_get_vn_id_no_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when no name is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_vn_id,
                          None,
                          call='foo')

    @patch('salt.cloud.clouds.opennebula.list_vns',
           MagicMock(return_value={'test-vn': {'id': '100'}}))
    def test_get_vn_id_not_found(self):
        '''
        Tests that a SaltCloudSystemExit is raised when no name is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.get_vn_id,
                          kwargs={'name': 'test'},
                          call='function')

    @patch('salt.cloud.clouds.opennebula.list_vns',
           MagicMock(return_value={'test-vn': {'id': '100'}}))
    def test_get_vn_id_success(self):
        '''
        Tests that the function returns successfully.
        '''
        mock_id = '100'
        mock_kwargs = {'name': 'test-vn'}
        self.assertEqual(opennebula.get_vn_id(mock_kwargs, 'foo'),
                         mock_id)

    # TODO: Write tests for create function

    def test_destroy_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.destroy, 'my-vm', 'function')

    def test_image_allocate_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.image_allocate, 'foo')

    def test_image_allocate_no_name_or_datastore_id(self):
        '''
        Tests that a SaltCloudSystemExit is raised when a neither a datastore_id
        nor a datastore_name is provided.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.image_allocate, 'function')

    def test_image_allocate_no_path_or_data(self):
        '''
        Tests that a SaltCloudSystemExit is raised when neither the path nor data args
        are provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.image_allocate,
                          'function',
                          kwargs={'datastore_id': '5'})

    def test_image_clone_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.image_clone, 'foo')

    def test_image_clone_no_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when a name isn't provided.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.image_clone, 'function')

    def test_image_clone_no_image_id_or_image_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when neither the image_id nor
        the image_name args are provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.image_clone,
                          'function',
                          kwargs={'name': 'test'})

    @skipIf(True, 'Need to figure out how to mock calls to the O.N. API first.')
    @patch('image.clone', MagicMock(return_value=[True, 11, 0]))
    def test_image_clone_success(self):
        '''
        Tests that image_clone returns successfully
        '''
        name = 'test-image'
        expected = {
            'action': 'image.clone',
            'cloned': 'True',
            'cloned_image_id': '11',
            'cloned_image_name': name,
            'error_code': '0',
        }
        ret = opennebula.image_clone('function', kwargs={'name': name, 'image_id': 1})
        self.assertEqual(expected, ret)

    def test_image_delete_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.image_delete, 'foo')

    def test_image_delete_no_name_or_image_id(self):
        '''
        Tests that a SaltCloudSystemExit is raised when a neither an image_id
        nor a name is provided.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.image_delete, 'function')

    def test_image_info_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.image_info, 'foo')

    def test_image_info_no_image_id_or_image_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when a neither an image_id
        nor a name is provided.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.image_info, 'function')

    def test_image_persist_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.image_persistent, 'foo')

    def test_image_persist_no_persist(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the persist kwarg is missing.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.image_persistent, 'function')

    def test_image_persist_no_name_or_image_id(self):
        '''
        Tests that a SaltCloudSystemExit is raised when a neither an image_id
        nor a name is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.image_delete,
                          'function',
                          kwargs={'persist': False})

    def test_image_snapshot_delete_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.image_snapshot_delete,
                          call='foo')

    def test_image_snapshot_delete_no_snapshot_id(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the snapshot_id kwarg is
        missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.image_snapshot_delete,
                          call='function',
                          kwargs=None)

    def test_image_snapshot_delete_no_image_name_or_image_id(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the image_id and image_name
        kwargs are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.image_snapshot_delete,
                          call='function',
                          kwargs={'snapshot_id': 0})

    def test_image_snapshot_revert_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.image_snapshot_revert,
                          call='foo')

    def test_image_snapshot_revert_no_snapshot_id(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the snapshot_id kwarg is
        missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.image_snapshot_revert,
                          call='function',
                          kwargs=None)

    def test_image_snapshot_revert_no_image_name_or_image_id(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the image_id and image_name
        kwargs are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.image_snapshot_revert,
                          call='function',
                          kwargs={'snapshot_id': 0})

    def test_image_snapshot_flatten_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.image_snapshot_flatten,
                          call='foo')

    def test_image_snapshot_flatten_no_snapshot_id(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the snapshot_id kwarg is
        missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.image_snapshot_flatten,
                          call='function',
                          kwargs=None)

    def test_image_snapshot_flatten_no_image_name_or_image_id(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the image_id and image_name
        kwargs are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.image_snapshot_flatten,
                          call='function',
                          kwargs={'snapshot_id': 0})

    def test_image_update_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.image_update, 'foo')

    def test_image_update_no_update_type(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the update_type kwarg is
        missing.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.image_update, 'function')

    def test_image_update_bad_update_type_value(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the update_type kwarg is
        not a valid value.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.image_update,
                          'function',
                          kwargs={'update_type': 'foo'})

    def test_image_update_no_image_id_or_image_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the image_id and image_name
        kwargs are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.image_update,
                          'function',
                          kwargs={'update_type': 'merge'})

    def test_image_update_no_data_or_path(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the data and path
        kwargs are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.image_update,
                          'function',
                          kwargs={'update_type': 'merge', 'image_id': '0'})

    def test_show_instance_action_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --action or -a is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.show_instance,
                          VM_NAME,
                          call='foo')

    @patch('salt.cloud.clouds.opennebula._get_node',
           MagicMock(return_value={'my-vm': {'name': 'my-vm', 'id': 0}}))
    def test_show_instance_success(self):
        '''
        Tests that the node was found successfully.
        '''
        ret = {'my-vm': {'name': 'my-vm', 'id': 0}}
        self.assertEqual(opennebula.show_instance('my-vm', call='action'), ret)

    def test_secgroup_allocate_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.secgroup_allocate, 'foo')

    def test_secgroup_allocate_no_data_or_path(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the data and path
        kwargs are missing.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.secgroup_allocate, 'function')

    def test_secgroup_clone_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.secgroup_clone, 'foo')

    def test_secgroup_clone_no_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the name kwarg is
        missing.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.secgroup_clone, 'function')

    def test_secgroup_clone_no_secgroup_id_or_secgroup_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the secgroup_id and
        secgroup_name kwargs are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.secgroup_clone,
                          'function',
                          kwargs={'name': 'test'})

    def test_secgroup_delete_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.secgroup_delete, 'foo')

    def test_secgroup_delete_no_secgroup_id_or_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the secgroup_id and
        name kwargs are missing.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.secgroup_clone, 'function')

    def test_secgroup_info_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.secgroup_info, 'foo')

    def test_secgroup_info_no_secgroup_id_or_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the secgroup_id and
        name kwargs are missing.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.secgroup_info, 'function')

    def test_secgroup_update_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.secgroup_update, 'foo')

    def test_secgroup_update_no_update_type(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the update_type arg is
        missing.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.secgroup_update, 'function')

    def test_secgroup_update_bad_update_type_value(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the update_type contains
        an invalid value.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.secgroup_update,
                          'function',
                          kwargs={'update_type': 'foo'})

    def test_secgroup_update_no_secgroup_id_or_secgroup_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the secgroup_id and
        secgroup_name kwargs are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.secgroup_update,
                          'function',
                          kwargs={'update_type': 'merge'})

    def test_secgroup_update_no_data_or_path(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the data and
        path kwargs are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.secgroup_update,
                          'function',
                          kwargs={'update_type': 'merge', 'secgroup_id': '0'})

    def test_template_allocate_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.template_allocate, 'foo')

    def test_template_allocate_no_data_or_path(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the data and
        path kwargs are missing.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.template_allocate, 'function')

    def test_template_clone_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.template_clone, 'foo')

    def test_template_clone_no_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the name arg is missing.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.template_clone, 'function')

    def test_template_clone_no_template_name_or_template_id(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the template_name and
        template_id args are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.template_clone,
                          'function',
                          kwargs={'name': 'foo'})

    def test_template_delete_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.template_delete, 'foo')

    def test_template_delete_no_name_or_template_id(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the name and
        template_id args are missing.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.template_delete, 'function')

    def test_template_instantiate_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.template_instantiate, 'foo')

    def test_template_instantiate_no_vm_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the vm_name arg is missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.template_instantiate,
                          'function',
                          None)

    def test_template_instantiate_no_template_id_or_template_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the template_name and
        template_id args are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.template_instantiate,
                          'function',
                          kwargs={'vm_name': 'test'})

    def test_template_update_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.template_update,
                          call='foo')

    def test_template_update_bad_update_type_value(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the update_type contains
        and invalid value.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.template_update,
                          call='function',
                          kwargs={'update_type': 'foo'})

    def test_template_update_no_template_id_or_template_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the template_id and the
        template_name args are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.template_update,
                          call='function',
                          kwargs={'update_type': 'merge'})

    def test_template_update_no_data_or_path(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the data and the
        path args are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.template_update,
                          call='function',
                          kwargs={'update_type': 'merge',
                                  'template_id': '0'})

    def test_vm_action_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --action or -a is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_action,
                          VM_NAME,
                          call='foo')

    def test_vm_action_no_action(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the action arg is missing
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_action,
                          VM_NAME,
                          call='action')

    def test_vm_allocate_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.vm_allocate, 'foo')

    def test_vm_allocate_no_data_or_path(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the data and
        path kwargs are missing.
        '''
        self.assertRaises(SaltCloudSystemExit, opennebula.vm_allocate, 'function')

    def test_vm_attach_action_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --action or -a is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_attach,
                          VM_NAME,
                          call='foo')

    def test_vm_attach_no_data_or_path(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the data and
        path kwargs are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_attach,
                          VM_NAME,
                          call='action')

    def test_vm_attach_nic_action_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --action or -a is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_attach_nic,
                          VM_NAME,
                          call='foo')

    def test_vm_attach_nic_no_data_or_path(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the data and
        path kwargs are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_attach_nic,
                          VM_NAME,
                          call='action')

    def test_vm_deploy_action_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --action or -a is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_deploy,
                          VM_NAME,
                          call='foo')

    def test_vm_deploy_no_host_id_or_host_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the host_id and the
        host_name args are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_deploy,
                          VM_NAME,
                          call='action',
                          kwargs=None)

    def test_vm_detach_action_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --action or -a is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_detach,
                          VM_NAME,
                          call='foo')

    def test_vm_detach_no_disk_id(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the disk_id ar is missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_detach,
                          VM_NAME,
                          call='action')

    def test_vm_detach_nic_action_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --action or -a is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_detach_nic,
                          VM_NAME,
                          call='foo')

    def test_vm_detach_nic_no_nic_id(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the nic_id arg is missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_detach_nic,
                          VM_NAME,
                          call='action')

    def test_vm_disk_save_action_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --action or -a is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_disk_save,
                          VM_NAME,
                          call='foo')

    def test_vm_disk_save_no_disk_id(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the disk_id arg is missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_disk_save,
                          VM_NAME,
                          call='action',
                          kwargs={'image_name': 'foo'})

    def test_vm_disk_save_no_image_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the image_name arg is missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_disk_save,
                          VM_NAME,
                          call='action',
                          kwargs={'disk_id': '0'})

    def test_vm_disk_snapshot_create_action_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --action or -a is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_disk_snapshot_create,
                          VM_NAME,
                          call='foo')

    def test_vm_disk_snapshot_create_no_disk_id(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the disk_id arg is missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_disk_snapshot_create,
                          VM_NAME,
                          call='action',
                          kwargs={'description': 'foo'})

    def test_vm_disk_snapshot_create_no_description(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the image_name arg is missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_disk_snapshot_create,
                          VM_NAME,
                          call='action',
                          kwargs={'disk_id': '0'})

    def test_vm_disk_snapshot_delete_action_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --action or -a is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_disk_snapshot_delete,
                          VM_NAME,
                          call='foo')

    def test_vm_disk_snapshot_delete_no_disk_id(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the disk_id arg is missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_disk_snapshot_delete,
                          VM_NAME,
                          call='action',
                          kwargs={'snapshot_id': '0'})

    def test_vm_disk_snapshot_delete_no_snapshot_id(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the snapshot_id arg is missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_disk_snapshot_delete,
                          VM_NAME,
                          call='action',
                          kwargs={'disk_id': '0'})

    def test_vm_disk_snapshot_revert_action_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --action or -a is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_disk_snapshot_revert,
                          VM_NAME,
                          call='foo')

    def test_vm_disk_snapshot_revert_no_disk_id(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the disk_id arg is missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_disk_snapshot_revert,
                          VM_NAME,
                          call='action',
                          kwargs={'snapshot_id': '0'})

    def test_vm_disk_snapshot_revert_no_snapshot_id(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the snapshot_id arg is missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_disk_snapshot_revert,
                          VM_NAME,
                          call='action',
                          kwargs={'disk_id': '0'})

    def test_vm_info_action_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --action or -a is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_info,
                          VM_NAME,
                          call='foo')

    def test_vm_migrate_action_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --action or -a is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_migrate,
                          VM_NAME,
                          call='foo')

    def test_vm_migrate_no_datastore_id_or_datastore_name(self):
        '''
        Tests that a SaltCLoudSystemExit is raised when the datastore_id and the
        datastore_name args are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_migrate,
                          VM_NAME,
                          call='action',
                          kwargs=None)

    def test_vm_migrate_no_host_id_or_host_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the the host_id and the
        host_name args are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_migrate,
                          VM_NAME,
                          call='action',
                          kwargs={'datastore_id': '0'})

    def test_vm_monitoring_action_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --action or -a is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_monitoring,
                          VM_NAME,
                          call='foo')

    def test_vm_resize_action_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --action or -a is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_resize,
                          VM_NAME,
                          call='foo')

    def test_vm_resize_no_data_or_path(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the data and path args
        are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_resize,
                          VM_NAME,
                          call='action',
                          kwargs=None)

    def test_vm_snapshot_create_action_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --action or -a is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_snapshot_create,
                          VM_NAME,
                          call='foo')

    def test_vm_snapshot_create_no_snapshot_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the snapshot_name arg
        is missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_snapshot_create,
                          VM_NAME,
                          call='action',
                          kwargs=None)

    def test_vm_snapshot_delete_action_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --action or -a is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_snapshot_delete,
                          VM_NAME,
                          call='foo')

    def test_vm_snapshot_delete_no_snapshot_id(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the snapshot_id arg
        is missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_snapshot_delete,
                          VM_NAME,
                          call='action',
                          kwargs=None)

    def test_vm_snapshot_revert_action_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --action or -a is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_snapshot_revert,
                          VM_NAME,
                          call='foo')

    def test_vm_snapshot_revert_no_snapshot_id(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the snapshot_id arg
        is missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_snapshot_revert,
                          VM_NAME,
                          call='action',
                          kwargs=None)

    def test_vm_update_action_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --action or -a is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_update,
                          VM_NAME,
                          call='foo')

    def test_vm_update_no_update_type(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the update_type arg
        is missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_update,
                          VM_NAME,
                          call='action',
                          kwargs=None)

    def test_vm_update_bad_update_type_value(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the update_type kwarg is
        not a valid value.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_update,
                          VM_NAME,
                          call='action',
                          kwargs={'update_type': 'foo'})

    def test_vm_update_no_data_or_path(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the data and path args
        are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vm_update,
                          VM_NAME,
                          call='action',
                          kwargs={'update_type': 'merge'})

    def test_vn_add_ar_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vn_add_ar,
                          call='foo')

    def test_vn_add_ar_no_vn_id_or_vn_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the vn_id and vn_name
        args are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vn_add_ar,
                          call='function',
                          kwargs=None)

    def test_vn_add_ar_no_path_or_data(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the path and data
        args are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vn_add_ar,
                          call='function',
                          kwargs={'vn_id': '0'})

    def test_vn_allocate_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vn_allocate,
                          call='foo')

    def test_vn_allocate_no_data_or_path(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the path and data
        args are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vn_allocate,
                          call='function',
                          kwargs=None)

    def test_vn_delete_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vn_delete,
                          call='foo')

    def test_vn_delete_no_vn_id_or_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the vn_id and name
        args are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vn_delete,
                          call='function',
                          kwargs=None)

    def test_vn_free_ar_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vn_free_ar,
                          call='foo')

    def test_vn_free_ar_no_ar_id(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the ar_id is missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vn_free_ar,
                          call='function',
                          kwargs=None)

    def test_vn_free_ar_no_vn_id_or_vn_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the vn_id and vn_name
        args are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vn_free_ar,
                          call='function',
                          kwargs={'ar_id': '0'})

    def test_vn_hold_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vn_hold,
                          call='foo')

    def test_vn_hold_no_vn_id_or_vn_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the vn_id and vn_name
        args are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vn_hold,
                          call='function',
                          kwargs=None)

    def test_vn_hold_no_data_or_path(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the data and path
        args are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vn_hold,
                          call='function',
                          kwargs={'vn_id': '0'})

    def test_vn_info_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vn_info,
                          call='foo')

    def test_vn_info_no_vn_id_or_vn_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the vn_id and vn_name
        args are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vn_info,
                          call='function',
                          kwargs=None)

    def test_vn_release_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vn_release,
                          call='foo')

    def test_vn_release_no_vn_id_or_vn_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the vn_id and vn_name
        args are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vn_release,
                          call='function',
                          kwargs=None)

    def test_vn_release_no_data_or_path(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the data and path
        args are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vn_release,
                          call='function',
                          kwargs={'vn_id': '0'})

    def test_vn_reserve_function_error(self):
        '''
        Tests that a SaltCloudSystemExit is raised when something other than
        --function or -f is provided.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vn_reserve,
                          call='foo')

    def test_vn_reserve_no_vn_id_or_vn_name(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the vn_id and vn_name
        args are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vn_reserve,
                          call='function',
                          kwargs=None)

    def test_vn_reserve_no_data_or_path(self):
        '''
        Tests that a SaltCloudSystemExit is raised when the data and path
        args are missing.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula.vn_reserve,
                          call='function',
                          kwargs={'vn_id': '0'})

    @skipIf(not HAS_XML_LIBS, 'cannot find lxml python library')
    def test__get_xml(self):
        '''
        Tests that invalid XML raises SaltCloudSystemExit.
        '''
        self.assertRaises(SaltCloudSystemExit,
                          opennebula._get_xml,
                          "[VirtualMachinePoolInfo] User couldn't be"
                          " authenticated, aborting call.")


if __name__ == '__main__':
    from integration import run_tests
    run_tests(OpenNebulaTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Direct call executor module

@author: Dmitry Kuzmenko <dmitry.kuzmenko@dsr-company.com>
'''
from __future__ import absolute_import
from salt.executors import ModuleExecutorBase


def get(*args, **kwargs):
    return DirectCallExecutor(*args, **kwargs)


class DirectCallExecutor(ModuleExecutorBase):
    '''
    Directly calls the given function with arguments
    '''

    def __init__(self, opts, data, func, args, kwargs):
        '''
        Constructor
        '''
        super(DirectCallExecutor, self).__init__()
        self.func, self.args, self.kwargs = func, args, kwargs

    def execute(self):
        return self.func(*self.args, **self.kwargs)



# -*- coding: utf-8 -*-
'''
runit service module
(http://smarden.org/runit)

This module is compatible with the :mod:`service <salt.states.service>` states,
so it can be used to maintain services using the ``provider`` argument:

.. code-block:: yaml

    myservice:
      service:
        - running
        - provider: runit

Provides virtual `service` module on systems using runit as init.


Service management rules (`sv` command):

    service $n is ENABLED   if file SERVICE_DIR/$n/run exists
    service $n is AVAILABLE if ENABLED or if file AVAIL_SVR_DIR/$n/run exists
    service $n is DISABLED  if AVAILABLE but not ENABLED

    SERVICE_DIR/$n is normally a symlink to a AVAIL_SVR_DIR/$n folder


Service auto-start/stop mechanism:

    `sv` (auto)starts/stops service as soon as SERVICE_DIR/<service> is
    created/deleted, both on service creation or a boot time.

    autostart feature is disabled if file SERVICE_DIR/<n>/down exists. This
    does not affect the current's service status (if already running) nor
    manual service management.


Service's alias:

    Service `sva` is an alias of service `svc` when `AVAIL_SVR_DIR/sva` symlinks
    to folder `AVAIL_SVR_DIR/svc`. `svc` can't be enabled if it is already
    enabled through an alias already enabled, since `sv` files are stored in
    folder `SERVICE_DIR/svc/`.

    XBPS package management uses a service's alias to provides service
    alternative(s), such as chrony and openntpd both aliased to ntpd.
'''
from __future__ import absolute_import

# Import python libs
import os
import glob
import logging
import time

log = logging.getLogger(__name__)

# Import salt libs
from salt.exceptions import CommandExecutionError
import salt.utils

# Function alias to not shadow built-ins.
__func_alias__ = {
    'reload_': 'reload'
}

# which dir sv works with
VALID_SERVICE_DIRS = [
    '/service',
    '/var/service',
    '/etc/service',
]
SERVICE_DIR = None
for service_dir in VALID_SERVICE_DIRS:
    if os.path.exists(service_dir):
        SERVICE_DIR = service_dir
        break

# available service directory(ies)
AVAIL_SVR_DIRS = []

# Define the module's virtual name
__virtualname__ = 'service'


def __virtual__():
    '''
    Virtual service only on systems using runit as init process (PID 1).
    Otherwise, use this module with the provider mechanism.
    '''
    if __grains__.get('init') == 'runit':
        if __grains__['os'] == 'Void':
            add_svc_avail_path('/etc/sv')
        return __virtualname__
    return False


def _service_path(name):
    '''
    Return SERVICE_DIR+name if possible

    name
        the service's name to work on
    '''
    if not SERVICE_DIR:
        raise CommandExecutionError('Could not find service directory.')
    return os.path.join(SERVICE_DIR, name)


#-- states.service  compatible args
def start(name):
    '''
    Start service

    name
        the service's name

    CLI Example:

    .. code-block:: bash

        salt '*' runit.start <service name>
    '''
    cmd = 'sv start {0}'.format(_service_path(name))
    return not __salt__['cmd.retcode'](cmd)


#-- states.service compatible args
def stop(name):
    '''
    Stop service

    name
        the service's name

    CLI Example:

    .. code-block:: bash

        salt '*' runit.stop <service name>
    '''
    cmd = 'sv stop {0}'.format(_service_path(name))
    return not __salt__['cmd.retcode'](cmd)


#-- states.service compatible
def reload_(name):
    '''
    Reload service

    name
        the service's name

    CLI Example:

    .. code-block:: bash

        salt '*' runit.reload <service name>
    '''
    cmd = 'sv reload {0}'.format(_service_path(name))
    return not __salt__['cmd.retcode'](cmd)


#-- states.service compatible
def restart(name):
    '''
    Restart service

    name
        the service's name

    CLI Example:

    .. code-block:: bash

        salt '*' runit.restart <service name>
    '''
    cmd = 'sv restart {0}'.format(_service_path(name))
    return not __salt__['cmd.retcode'](cmd)


#-- states.service compatible
def full_restart(name):
    '''
    Calls runit.restart()

    name
        the service's name

    CLI Example:

    .. code-block:: bash

        salt '*' runit.full_restart <service name>
    '''
    restart(name)


#-- states.service compatible
def status(name, sig=None):
    '''
    Return ``True`` if service is running

    name
        the service's name

    sig
        signature to identify with ps

    CLI Example:

    .. code-block:: bash

        salt '*' runit.status <service name>
    '''
    if sig:
        # usual way to do by others (debian_service, netbsdservice).
        # XXX probably does not work here (check 'runsv sshd' instead of 'sshd' ?)
        return bool(__salt__['status.pid'](sig))

    svc_path = _service_path(name)
    if not os.path.exists(svc_path):
        # service does not exist
        return False

    # sv return code is not relevant to get a service status.
    # Check its output instead.
    cmd = 'sv status {0}'.format(svc_path)
    try:
        out = __salt__['cmd.run_stdout'](cmd)
        return out.startswith('run: ')
    except Exception:
        # sv (as a command) returned an error
        return False


def _is_svc(svc_path):
    '''
    Return ``True`` if directory <svc_path> is really a service:
    file <svc_path>/run exists and is executable

    svc_path
        the (absolute) directory to check for compatibility
    '''
    run_file = os.path.join(svc_path, 'run')
    if (os.path.exists(svc_path)
         and os.path.exists(run_file)
         and os.access(run_file, os.X_OK)):
        return True
    return False


def status_autostart(name):
    '''
    Return ``True`` if service <name> is autostarted by sv
    (file $service_folder/down does not exist)
    NB: return ``False`` if the service is not enabled.

    name
        the service's name

    CLI Example:

    .. code-block:: bash

        salt '*' runit.status_autostart <service name>
    '''
    return not os.path.exists(os.path.join(_service_path(name), 'down'))


def get_svc_broken_path(name='*'):
    '''
    Return list of broken path(s) in SERVICE_DIR that match ``name``

    A path is broken if it is a broken symlink or can not be a runit service

    name
        a glob for service name. default is '*'

    CLI Example:

    .. code-block:: bash

        salt '*' runit.get_svc_broken_path <service name>
    '''
    if not SERVICE_DIR:
        raise CommandExecutionError('Could not find service directory.')

    ret = set()

    for el in glob.glob(os.path.join(SERVICE_DIR, name)):
        if not _is_svc(el):
            ret.add(el)
    return sorted(ret)


def get_svc_avail_path():
    '''
    Return list of paths that may contain available services
    '''
    return AVAIL_SVR_DIRS


def add_svc_avail_path(path):
    '''
    Add a path that may contain available services.
    Return ``True`` if added (or already present), ``False`` on error.

    path
        directory to add to AVAIL_SVR_DIRS
    '''
    if os.path.exists(path):
        if path not in AVAIL_SVR_DIRS:
            AVAIL_SVR_DIRS.append(path)
        return True
    return False


def _get_svc_path(name='*', status=None):
    '''
    Return a list of paths to services with ``name`` that have the specified ``status``

    name
        a glob for service name. default is '*'

    status
        None       : all services (no filter, default choice)
        'DISABLED' : available service(s) that is not enabled
        'ENABLED'  : enabled service (whether started on boot or not)
    '''

    # This is the core routine to work with services, called by many
    # other functions of this module.
    #
    # The name of a service is the "apparent" folder's name that contains its
    # "run" script. If its "folder" is a symlink, the service is an "alias" of
    # the targeted service.

    if not SERVICE_DIR:
        raise CommandExecutionError('Could not find service directory.')

    # path list of enabled services as /AVAIL_SVR_DIRS/$service,
    # taking care of any service aliases (do not use os.path.realpath()).
    ena = set()
    for el in glob.glob(os.path.join(SERVICE_DIR, name)):
        if _is_svc(el):
            ena.add(os.readlink(el))
            log.trace('found enabled service path: {0}'.format(el))

    if status == 'ENABLED':
        return sorted(ena)

    # path list of available services as /AVAIL_SVR_DIRS/$service
    ava = set()
    for d in AVAIL_SVR_DIRS:
        for el in glob.glob(os.path.join(d, name)):
            if _is_svc(el):
                ava.add(el)
                log.trace('found available service path: {0}'.format(el))

    if status == 'DISABLED':
        # service available but not enabled
        ret = ava.difference(ena)
    else:
        # default: return available services
        ret = ava.union(ena)

    return sorted(ret)


def _get_svc_list(name='*', status=None):
    '''
    Return list of services that have the specified service ``status``

    name
        a glob for service name. default is '*'

    status
        None       : all services (no filter, default choice)
        'DISABLED' : available service that is not enabled
        'ENABLED'  : enabled service (whether started on boot or not)
    '''
    return sorted([os.path.basename(el) for el in _get_svc_path(name, status)])


def get_svc_alias():
    '''
    Returns the list of service's name that are aliased and their alias path(s)
    '''

    ret = {}
    for d in AVAIL_SVR_DIRS:
        for el in glob.glob(os.path.join(d, '*')):
            if not os.path.islink(el):
                continue
            psvc = os.readlink(el)
            if not os.path.isabs(psvc):
                psvc = os.path.join(d, psvc)
            nsvc = os.path.basename(psvc)
            if nsvc not in ret:
                ret[nsvc] = []
            ret[nsvc].append(el)
    return ret


def available(name):
    '''
    Returns ``True`` if the specified service is available, otherwise returns
    ``False``.

    name
        the service's name

    CLI Example:

    .. code-block:: bash

        salt '*' runit.available <service name>
    '''
    return name in _get_svc_list(name)


def missing(name):
    '''
    The inverse of runit.available.
    Returns ``True`` if the specified service is not available, otherwise returns
    ``False``.

    name
        the service's name

    CLI Example:

    .. code-block:: bash

        salt '*' runit.missing <service name>
    '''
    return name not in _get_svc_list(name)


def get_all():
    '''
    Return a list of all available services

    CLI Example:

    .. code-block:: bash

        salt '*' runit.get_all
    '''
    return _get_svc_list()


def get_enabled():
    '''
    Return a list of all enabled services

    CLI Example:

    .. code-block:: bash

        salt '*' service.get_enabled
    '''
    return _get_svc_list(status='ENABLED')


def get_disabled():
    '''
    Return a list of all disabled services

    CLI Example:

    .. code-block:: bash

        salt '*' service.get_disabled
    '''
    return _get_svc_list(status='DISABLED')


def enabled(name):
    '''
    Return ``True`` if the named service is enabled, ``False`` otherwise

    name
        the service's name

    CLI Example:

    .. code-block:: bash

        salt '*' service.enabled <service name>
    '''
    # exhaustive check instead of (only) os.path.exists(_service_path(name))
    return name in _get_svc_list(name, 'ENABLED')


def disabled(name):
    '''
    Return ``True`` if the named service is disabled, ``False``  otherwise

    name
        the service's name

    CLI Example:

    .. code-block:: bash

        salt '*' service.disabled <service name>
    '''
    # return True for a non-existent service
    return name not in _get_svc_list(name, 'ENABLED')


def show(name):
    '''
    Show properties of one or more units/jobs or the manager

    name
        the service's name

    CLI Example:

        salt '*' service.show <service name>
    '''
    ret = {}
    ret['enabled'] = False
    ret['disabled'] = True
    ret['running'] = False
    ret['service_path'] = None
    ret['autostart'] = False
    ret['command_path'] = None

    ret['available'] = available(name)
    if not ret['available']:
        return ret

    ret['enabled'] = enabled(name)
    ret['disabled'] = not ret['enabled']
    ret['running'] = status(name)
    ret['autostart'] = status_autostart(name)
    ret['service_path'] = _get_svc_path(name)[0]
    if ret['service_path']:
        ret['command_path'] = os.path.join(ret['service_path'], 'run')

    # XXX provide info about alias ?

    return ret


def enable(name, start=False, **kwargs):
    '''
    Start service ``name`` at boot.
    Returns ``True`` if operation is successful

    name
        the service's name

    start
        ``False`` : Do not start the service once enabled. Default mode.
                    (consistent with other service management)
        ``True``  : also start the service at the same time (default sv mode)

    CLI Example:

    .. code-block:: bash

        salt '*' service.enable <name> [start=True]
    '''

    # non-existent service
    if not available(name):
        return False

    # if service is aliased, refuse to enable it
    alias = get_svc_alias()
    if name in alias:
        log.error('This service is aliased, enable its alias instead')
        return False

    # down_file: file that disables sv autostart
    svc_realpath = _get_svc_path(name)[0]
    down_file = os.path.join(svc_realpath, 'down')

    # if service already enabled, remove down_file to
    # let service starts on boot (as requested)
    if enabled(name):
        if os.path.exists(down_file):
            try:
                os.unlink(down_file)
            except OSError:
                log.error('Unable to remove file {0}'.format(down_file))
                return False
        return True

    # let's enable the service

    if not start:
        # create a temp 'down' file BEFORE enabling service.
        # will prevent sv from starting this service automatically.
        log.trace('need a temporary file {0}'.format(down_file))
        if not os.path.exists(down_file):
            try:
                salt.utils.fopen(down_file, "w").close()
            except IOError:
                log.error('Unable to create file {0}'.format(down_file))
                return False

    # enable the service
    try:
        os.symlink(svc_realpath, _service_path(name))

    except IOError:
        # (attempt to) remove temp down_file anyway
        log.error('Unable to create symlink {0}'.format(down_file))
        if not start:
            os.unlink(down_file)
        return False

    # ensure sv is aware of this new service before continuing.
    # if not, down_file might be removed too quickly,
    # before 'sv' have time to take care about it.
    # Documentation indicates that a change is handled within 5 seconds.
    cmd = 'sv status {0}'.format(_service_path(name))
    retcode_sv = 1
    count_sv = 0
    while retcode_sv != 0 and count_sv < 10:
        time.sleep(0.5)
        count_sv += 1
        call = __salt__['cmd.run_all'](cmd)
        retcode_sv = call['retcode']

    # remove the temp down_file in any case.
    if (not start) and os.path.exists(down_file):
        try:
            os.unlink(down_file)
        except OSError:
            log.error('Unable to remove temp file {0}'.format(down_file))
            retcode_sv = 1

    # if an error happened, revert our changes
    if retcode_sv != 0:
        os.unlink(os.path.join([_service_path(name), name]))
        return False
    return True


def disable(name, stop=False, **kwargs):
    '''
    Don't start service ``name`` at boot
    Returns ``True`` if operation is successfull

    name
        the service's name

    stop
        if True, also stops the service

    CLI Example:

    .. code-block:: bash

        salt '*' service.disable <name> [stop=True]
    '''

    # non-existent as registrered service
    if not enabled(name):
        return False

    # down_file: file that prevent sv autostart
    svc_realpath = _get_svc_path(name)[0]
    down_file = os.path.join(svc_realpath, 'down')

    if stop:
        stop(name)

    if not os.path.exists(down_file):
        try:
            salt.utils.fopen(down_file, "w").close()
        except IOError:
            log.error('Unable to create file {0}'.format(down_file))
            return False

    return True


def remove(name):
    '''
    Remove the service <name> from system.
    Returns ``True`` if operation is successfull.
    The service will be also stopped.

    name
        the service's name

    CLI Example:

    .. code-block:: bash

        salt '*' service.remove <name>
    '''

    if not enabled(name):
        return False

    svc_path = _service_path(name)
    if not os.path.islink(svc_path):
        log.error('{0} is not a symlink: not removed'.format(svc_path))
        return False

    if not stop(name):
        log.error('Failed to stop service {0}'.format(name))
        return False
    try:
        os.remove(svc_path)
    except IOError:
        log.error('Unable to remove symlink {0}'.format(svc_path))
        return False
    return True


# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4



# -*- coding: utf-8 -*-
'''
Display return data in YAML format
==================================

This outputter defaults to printing in YAML block mode for better readability.

Example output::

    saltmine:
      foo:
        bar: baz
        dictionary:
          abc: 123
          def: 456
        list:
          - Hello
          - World
'''
from __future__ import absolute_import

# Import third party libs
import logging
import yaml

# Import salt libs
from salt.utils.yamldumper import OrderedDumper

# Define the module's virtual name
__virtualname__ = 'yaml'

log = logging.getLogger(__name__)


def __virtual__():
    return __virtualname__


def output(data, **kwargs):  # pylint: disable=unused-argument
    '''
    Print out YAML using the block mode
    '''

    params = dict(Dumper=OrderedDumper)
    if 'output_indent' not in __opts__:
        # default indentation
        params.update(default_flow_style=False)
    elif __opts__['output_indent'] >= 0:
        # custom indent
        params.update(default_flow_style=False,
                      indent=__opts__['output_indent'])
    else:  # no indentation
        params.update(default_flow_style=True,
                      indent=0)
    try:
        return yaml.dump(data, **params)
    except Exception as exc:
        import pprint
        log.exception('Exception {0} encountered when trying to serialize {1}'.format(
            exc, pprint.pformat(data)))



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rahul Handay <rahulha@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

ensure_in_syspath('../../')

# Import Salt Libs
from salt.states import mdadm

# Globals
mdadm.__salt__ = {}
mdadm.__opts__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class MdadmTestCase(TestCase):
    '''
        Validate the mdadm state
    '''
    def test_present(self):
        '''
            Test to verify that the raid is present
        '''
        ret = [{'changes': {}, 'comment': 'Raid salt already present',
                'name': 'salt', 'result': True},
               {'changes': {},
                'comment': "Devices are a mix of RAID constituents"
                " (['dev0']) and non-RAID-constituents(['dev1']).",
                'name': 'salt', 'result': False},
               {'changes': {},
                'comment': 'Raid will be created with: True', 'name': 'salt',
                'result': None},
               {'changes': {}, 'comment': 'Raid salt failed to be created.',
                'name': 'salt', 'result': False}]

        mock = MagicMock(side_effect=[{'salt': True}, {'salt': False},
                                      {'salt': False}, {'salt': False},
                                      {'salt': False}])
        with patch.dict(mdadm.__salt__, {'raid.list': mock}):
            self.assertEqual(mdadm.present("salt", 5, "dev0"), ret[0])

            mock = MagicMock(side_effect=[0, 1])
            with patch.dict(mdadm.__salt__, {'cmd.retcode': mock}):
                self.assertDictEqual(mdadm.present("salt", 5,
                                                   ["dev0", "dev1"]),
                                     ret[1])

            mock = MagicMock(return_value=True)
            with patch.dict(mdadm.__salt__, {'cmd.retcode': mock}):
                with patch.dict(mdadm.__opts__, {'test': True}):
                    with patch.dict(mdadm.__salt__, {'raid.create': mock}):
                        self.assertDictEqual(mdadm.present("salt", 5, "dev0"),
                                             ret[2])

                with patch.dict(mdadm.__opts__, {'test': False}):
                    with patch.dict(mdadm.__salt__, {'raid.create': mock}):
                        self.assertDictEqual(mdadm.present("salt", 5, "dev0"),
                                             ret[3])

    def test_absent(self):
        '''
            Test to verify that the raid is absent
        '''
        ret = [{'changes': {}, 'comment': 'Raid salt already absent',
                'name': 'salt', 'result': True},
               {'changes': {},
                'comment': 'Raid saltstack is set to be destroyed',
                'name': 'saltstack', 'result': None},
               {'changes': {}, 'comment': 'Raid saltstack has been destroyed',
                'name': 'saltstack', 'result': True}]

        mock = MagicMock(return_value=["saltstack"])
        with patch.dict(mdadm.__salt__, {'raid.list': mock}):
            self.assertDictEqual(mdadm.absent("salt"), ret[0])

            with patch.dict(mdadm.__opts__, {'test': True}):
                self.assertDictEqual(mdadm.absent("saltstack"), ret[1])

            with patch.dict(mdadm.__opts__, {'test': False}):
                mock = MagicMock(return_value=True)
                with patch.dict(mdadm.__salt__, {'raid.destroy': mock}):
                    self.assertDictEqual(mdadm.absent("saltstack"), ret[2])


if __name__ == '__main__':
    from integration import run_tests
    run_tests(MdadmTestCase, needs_daemon=False)
